/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_COLORMAP_SELECTION_H__
#define __PIKA_COLORMAP_SELECTION_H__


#define PIKA_TYPE_COLORMAP_SELECTION            (pika_colormap_selection_get_type ())
#define PIKA_COLORMAP_SELECTION(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_COLORMAP_SELECTION, PikaColormapSelection))
#define PIKA_COLORMAP_SELECTION_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_COLORMAP_SELECTION, PikaColormapSelectionClass))
#define PIKA_IS_COLORMAP_SELECTION(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_COLORMAP_SELECTION))
#define PIKA_IS_COLORMAP_SELECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_COLORMAP_SELECTION))
#define PIKA_COLORMAP_SELECTION_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_COLORMAP_SELECTION, PikaColormapSelectionClass))


typedef struct _PikaColormapSelectionClass PikaColormapSelectionClass;

struct _PikaColormapSelection
{
  GtkBox  parent_instance;

  PikaContext     *context;
  PikaImage       *active_image;
  PikaPalette     *active_palette;

  GtkWidget       *view;
  gint             col_index;

  PangoLayout     *layout;

  GtkAdjustment   *index_adjustment;
  GtkWidget       *index_spinbutton;
  GtkWidget       *color_entry;

  GtkWidget       *color_dialog;
  GtkWidget       *right_vbox;
};

struct _PikaColormapSelectionClass
{
  GtkBoxClass  parent_class;

  void (* color_clicked)   (PikaColormapSelection *selection,
                            PikaPaletteEntry      *entry,
                            GdkModifierType        state);
  void (* color_activated) (PikaColormapSelection *selection,
                            PikaPaletteEntry      *entry);
};


GType       pika_colormap_selection_get_type   (void) G_GNUC_CONST;

GtkWidget * pika_colormap_selection_new        (PikaContext *context);

gint        pika_colormap_selection_get_index  (PikaColormapSelection *selection,
                                                const PikaRGB         *search);
gboolean    pika_colormap_selection_set_index  (PikaColormapSelection *selection,
                                                gint                   index,
                                                PikaRGB               *color);

gint        pika_colormap_selection_max_index  (PikaColormapSelection *selection);

PikaPaletteEntry * pika_colormap_selection_get_selected_entry (PikaColormapSelection *selection);

void               pika_colormap_selection_get_entry_rect (PikaColormapSelection *selection,
                                                           PikaPaletteEntry      *entry,
                                                           GdkRectangle          *rect);

#endif /* __PIKA_COLORMAP_SELECTION_H__ */

